# ===== THIS FILE IS GENERATED FROM A TEMPLATE ===== #
# ============== DO NOT EDIT DIRECTLY ============== #

from typing import TYPE_CHECKING, List, Optional
from ..call import call, call_async, call_sync

from ..dto import requests as dto
from .device_settings import DeviceSettings
from .axis import Axis
from .all_axes import AllAxes
from .warnings import Warnings
from ..dto.ascii.device_identity import DeviceIdentity
from .device_io import DeviceIO
from ..dto.ascii.response import Response
from .lockstep import Lockstep
from .oscilloscope import Oscilloscope
from .storage import DeviceStorage
from ..dto.ascii.can_set_state_device_response import CanSetStateDeviceResponse
from .pvt import Pvt
from .triggers import Triggers
from .streams import Streams
from ..dto.firmware_version import FirmwareVersion
from ..dto.measurement import Measurement
from ..dto.ascii.set_state_device_response import SetStateDeviceResponse

if TYPE_CHECKING:
    from .connection import Connection


class Device:
    """
    Represents the controller part of one device - may be either a standalone controller or an integrated controller.
    """

    @property
    def connection(self) -> 'Connection':
        """
        Connection of this device.
        """
        return self._connection

    @property
    def device_address(self) -> int:
        """
        The device address uniquely identifies the device on the connection.
        It can be configured or automatically assigned by the renumber command.
        """
        return self._device_address

    @property
    def settings(self) -> DeviceSettings:
        """
        Settings and properties of this device.
        """
        return self._settings

    @property
    def storage(self) -> DeviceStorage:
        """
        Key-value storage of this device.
        """
        return self._storage

    @property
    def io(self) -> DeviceIO:
        """
        I/O channels of this device.
        """
        return self._io

    @property
    def all_axes(self) -> AllAxes:
        """
        Virtual axis which allows you to target all axes of this device.
        """
        return self._all_axes

    @property
    def warnings(self) -> Warnings:
        """
        Warnings and faults of this device and all its axes.
        """
        return self._warnings

    @property
    def identity(self) -> DeviceIdentity:
        """
        Identity of the device.
        """
        return self.__retrieve_identity()

    @property
    def is_identified(self) -> bool:
        """
        Indicates whether or not the device has been identified.
        """
        return self.__retrieve_is_identified()

    @property
    def oscilloscope(self) -> Oscilloscope:
        """
        Oscilloscope recording helper for this device.
        Requires at least Firmware 7.00.
        """
        return self._oscilloscope

    @property
    def device_id(self) -> int:
        """
        Unique ID of the device hardware.
        """
        return self.identity.device_id

    @property
    def serial_number(self) -> int:
        """
        Serial number of the device.
        """
        return self.identity.serial_number

    @property
    def name(self) -> str:
        """
        Name of the product.
        """
        return self.identity.name

    @property
    def axis_count(self) -> int:
        """
        Number of axes this device has.
        """
        return self.identity.axis_count

    @property
    def firmware_version(self) -> FirmwareVersion:
        """
        Version of the firmware.
        """
        return self.identity.firmware_version

    @property
    def is_integrated(self) -> bool:
        """
        The device is an integrated product.
        """
        return self.identity.is_integrated

    @property
    def label(self) -> str:
        """
        User-assigned label of the device.
        """
        return self.__retrieve_label()

    @property
    def triggers(self) -> Triggers:
        """
        Triggers for this device.
        Requires at least Firmware 7.06.
        """
        return self._triggers

    @property
    def streams(self) -> Streams:
        """
        Gets an object that provides access to Streams on this device.
        Requires at least Firmware 7.05.
        """
        return self._streams

    @property
    def pvt(self) -> Pvt:
        """
        Gets an object that provides access to PVT functions of this device.
        Note that as of ZML v5.0.0, this returns a Pvt object and NOT a PvtSequence object.
        The PvtSequence can now be obtained from the Pvt object.
        Requires at least Firmware 7.33.
        """
        return self._pvt

    def __init__(self, connection: 'Connection', device_address: int):
        self._connection = connection
        self._device_address = device_address
        self._settings = DeviceSettings(self)
        self._storage = DeviceStorage(self)
        self._io = DeviceIO(self)
        self._all_axes = AllAxes(self)
        self._warnings = Warnings(self, 0)
        self._oscilloscope = Oscilloscope(self)
        self._triggers = Triggers(self)
        self._streams = Streams(self)
        self._pvt = Pvt(self)

    def identify(
            self,
            assume_version: Optional[FirmwareVersion] = None
    ) -> DeviceIdentity:
        """
        Queries the device and the database, gathering information about the product.
        Without this information features such as unit conversions will not work.
        Usually, called automatically by detect devices method.

        Args:
            assume_version: The identification assumes the specified firmware version
                instead of the version queried from the device.
                Providing this argument can lead to unexpected compatibility issues.

        Returns:
            Device identification data.
        """
        request = dto.DeviceIdentifyRequest(
            interface_id=self.connection.interface_id,
            device=self.device_address,
            assume_version=assume_version,
        )
        response = call(
            "device/identify",
            request,
            DeviceIdentity.from_binary)
        return response

    async def identify_async(
            self,
            assume_version: Optional[FirmwareVersion] = None
    ) -> DeviceIdentity:
        """
        Queries the device and the database, gathering information about the product.
        Without this information features such as unit conversions will not work.
        Usually, called automatically by detect devices method.

        Args:
            assume_version: The identification assumes the specified firmware version
                instead of the version queried from the device.
                Providing this argument can lead to unexpected compatibility issues.

        Returns:
            Device identification data.
        """
        request = dto.DeviceIdentifyRequest(
            interface_id=self.connection.interface_id,
            device=self.device_address,
            assume_version=assume_version,
        )
        response = await call_async(
            "device/identify",
            request,
            DeviceIdentity.from_binary)
        return response

    def generic_command(
            self,
            command: str,
            axis: int = 0,
            check_errors: bool = True,
            timeout: int = 0
    ) -> Response:
        """
        Sends a generic ASCII command to this device.
        For more information refer to: [ASCII Protocol Manual](https://www.zaber.com/protocol-manual#topic_commands).

        Args:
            command: Command and its parameters.
            axis: Optional axis number to send the command to.
            check_errors: Controls whether to throw an exception when the device rejects the command.
            timeout: The timeout, in milliseconds, for a device to respond to the command.
                Overrides the connection default request timeout.

        Returns:
            A response to the command.
        """
        request = dto.GenericCommandRequest(
            interface_id=self.connection.interface_id,
            device=self.device_address,
            command=command,
            axis=axis,
            check_errors=check_errors,
            timeout=timeout,
        )
        response = call(
            "interface/generic_command",
            request,
            Response.from_binary)
        return response

    async def generic_command_async(
            self,
            command: str,
            axis: int = 0,
            check_errors: bool = True,
            timeout: int = 0
    ) -> Response:
        """
        Sends a generic ASCII command to this device.
        For more information refer to: [ASCII Protocol Manual](https://www.zaber.com/protocol-manual#topic_commands).

        Args:
            command: Command and its parameters.
            axis: Optional axis number to send the command to.
            check_errors: Controls whether to throw an exception when the device rejects the command.
            timeout: The timeout, in milliseconds, for a device to respond to the command.
                Overrides the connection default request timeout.

        Returns:
            A response to the command.
        """
        request = dto.GenericCommandRequest(
            interface_id=self.connection.interface_id,
            device=self.device_address,
            command=command,
            axis=axis,
            check_errors=check_errors,
            timeout=timeout,
        )
        response = await call_async(
            "interface/generic_command",
            request,
            Response.from_binary)
        return response

    def generic_command_multi_response(
            self,
            command: str,
            axis: int = 0,
            check_errors: bool = True,
            timeout: int = 0
    ) -> List[Response]:
        """
        Sends a generic ASCII command to this device and expect multiple responses.
        Responses are returned in order of arrival.
        For more information refer to: [ASCII Protocol Manual](https://www.zaber.com/protocol-manual#topic_commands).

        Args:
            command: Command and its parameters.
            axis: Optional axis number to send the command to.
            check_errors: Controls whether to throw an exception when a device rejects the command.
            timeout: The timeout, in milliseconds, for a device to respond to the command.
                Overrides the connection default request timeout.

        Returns:
            All responses to the command.
        """
        request = dto.GenericCommandRequest(
            interface_id=self.connection.interface_id,
            device=self.device_address,
            command=command,
            axis=axis,
            check_errors=check_errors,
            timeout=timeout,
        )
        response = call(
            "interface/generic_command_multi_response",
            request,
            dto.GenericCommandResponseCollection.from_binary)
        return response.responses

    async def generic_command_multi_response_async(
            self,
            command: str,
            axis: int = 0,
            check_errors: bool = True,
            timeout: int = 0
    ) -> List[Response]:
        """
        Sends a generic ASCII command to this device and expect multiple responses.
        Responses are returned in order of arrival.
        For more information refer to: [ASCII Protocol Manual](https://www.zaber.com/protocol-manual#topic_commands).

        Args:
            command: Command and its parameters.
            axis: Optional axis number to send the command to.
            check_errors: Controls whether to throw an exception when a device rejects the command.
            timeout: The timeout, in milliseconds, for a device to respond to the command.
                Overrides the connection default request timeout.

        Returns:
            All responses to the command.
        """
        request = dto.GenericCommandRequest(
            interface_id=self.connection.interface_id,
            device=self.device_address,
            command=command,
            axis=axis,
            check_errors=check_errors,
            timeout=timeout,
        )
        response = await call_async(
            "interface/generic_command_multi_response",
            request,
            dto.GenericCommandResponseCollection.from_binary)
        return response.responses

    def generic_command_no_response(
            self,
            command: str,
            axis: int = 0
    ) -> None:
        """
        Sends a generic ASCII command to this device without expecting a response and without adding a message ID
        For more information refer to: [ASCII Protocol Manual](https://www.zaber.com/protocol-manual#topic_commands).

        Args:
            command: Command and its parameters.
            axis: Optional axis number to send the command to.
                Specifying -1 omits the number completely.
        """
        request = dto.GenericCommandRequest(
            interface_id=self.connection.interface_id,
            device=self.device_address,
            command=command,
            axis=axis,
        )
        call("interface/generic_command_no_response", request)

    async def generic_command_no_response_async(
            self,
            command: str,
            axis: int = 0
    ) -> None:
        """
        Sends a generic ASCII command to this device without expecting a response and without adding a message ID
        For more information refer to: [ASCII Protocol Manual](https://www.zaber.com/protocol-manual#topic_commands).

        Args:
            command: Command and its parameters.
            axis: Optional axis number to send the command to.
                Specifying -1 omits the number completely.
        """
        request = dto.GenericCommandRequest(
            interface_id=self.connection.interface_id,
            device=self.device_address,
            command=command,
            axis=axis,
        )
        await call_async("interface/generic_command_no_response", request)

    def get_axis(
            self,
            axis_number: int
    ) -> Axis:
        """
        Gets an Axis class instance which allows you to control a particular axis on this device.
        Axes are numbered from 1.

        Args:
            axis_number: Number of axis intended to control.

        Returns:
            Axis instance.
        """
        if axis_number <= 0:
            raise ValueError('Invalid value; physical axes are numbered from 1.')

        return Axis(self, axis_number)

    def get_lockstep(
            self,
            lockstep_group_id: int
    ) -> Lockstep:
        """
        Gets a Lockstep class instance which allows you to control a particular lockstep group on the device.
        Requires at least Firmware 6.15 or 7.11.

        Args:
            lockstep_group_id: The ID of the lockstep group to control. Lockstep group IDs start at one.

        Returns:
            Lockstep instance.
        """
        if lockstep_group_id <= 0:
            raise ValueError('Invalid value; lockstep groups are numbered from 1.')

        return Lockstep(self, lockstep_group_id)

    def prepare_command(
            self,
            command_template: str,
            *parameters: Measurement
    ) -> str:
        """
        Formats parameters into a command and performs unit conversions.
        Parameters in the command template are denoted by a question mark.
        Command returned is only valid for this device.
        Unit conversion is not supported for commands where axes can be remapped, such as stream and PVT commands.
        For more information refer to: [ASCII Protocol Manual](https://www.zaber.com/protocol-manual#topic_commands).

        Args:
            command_template: Template of a command to prepare. Parameters are denoted by question marks.
            parameters: Variable number of command parameters.

        Returns:
            Command with converted parameters.
        """
        request = dto.PrepareCommandRequest(
            interface_id=self.connection.interface_id,
            device=self.device_address,
            command_template=command_template,
            parameters=list(parameters),
        )
        response = call_sync(
            "device/prepare_command",
            request,
            dto.StringResponse.from_binary)
        return response.value

    def set_label(
            self,
            label: str
    ) -> None:
        """
        Sets the user-assigned device label.
        The label is stored on the controller and recognized by other software.

        Args:
            label: Label to set.
        """
        request = dto.DeviceSetStorageRequest(
            interface_id=self.connection.interface_id,
            device=self.device_address,
            value=label,
        )
        call("device/set_label", request)

    async def set_label_async(
            self,
            label: str
    ) -> None:
        """
        Sets the user-assigned device label.
        The label is stored on the controller and recognized by other software.

        Args:
            label: Label to set.
        """
        request = dto.DeviceSetStorageRequest(
            interface_id=self.connection.interface_id,
            device=self.device_address,
            value=label,
        )
        await call_async("device/set_label", request)

    def __retrieve_label(
            self
    ) -> str:
        """
        Gets the device name.

        Returns:
            The label.
        """
        request = dto.AxisEmptyRequest(
            interface_id=self.connection.interface_id,
            device=self.device_address,
        )
        response = call_sync(
            "device/get_label",
            request,
            dto.StringResponse.from_binary)
        return response.value

    def __repr__(
            self
    ) -> str:
        """
        Returns a string that represents the device.

        Returns:
            A string that represents the device.
        """
        request = dto.AxisToStringRequest(
            interface_id=self.connection.interface_id,
            device=self.device_address,
        )
        response = call_sync(
            "device/device_to_string",
            request,
            dto.StringResponse.from_binary)
        return response.value

    def get_state(
            self
    ) -> str:
        """
        Returns a serialization of the current device state that can be saved and reapplied.

        Returns:
            A serialization of the current state of the device.
        """
        request = dto.AxisEmptyRequest(
            interface_id=self.connection.interface_id,
            device=self.device_address,
        )
        response = call(
            "device/get_state",
            request,
            dto.StringResponse.from_binary)
        return response.value

    async def get_state_async(
            self
    ) -> str:
        """
        Returns a serialization of the current device state that can be saved and reapplied.

        Returns:
            A serialization of the current state of the device.
        """
        request = dto.AxisEmptyRequest(
            interface_id=self.connection.interface_id,
            device=self.device_address,
        )
        response = await call_async(
            "device/get_state",
            request,
            dto.StringResponse.from_binary)
        return response.value

    def set_state(
            self,
            state: str,
            device_only: bool = False
    ) -> SetStateDeviceResponse:
        """
        Applies a saved state to this device.

        Args:
            state: The state object to apply to this device.
            device_only: If true, only device scope settings and features will be set.

        Returns:
            Reports of any issues that were handled, but caused the state to not be exactly restored.
        """
        request = dto.SetStateRequest(
            interface_id=self.connection.interface_id,
            device=self.device_address,
            state=state,
            device_only=device_only,
        )
        response = call(
            "device/set_device_state",
            request,
            SetStateDeviceResponse.from_binary)
        return response

    async def set_state_async(
            self,
            state: str,
            device_only: bool = False
    ) -> SetStateDeviceResponse:
        """
        Applies a saved state to this device.

        Args:
            state: The state object to apply to this device.
            device_only: If true, only device scope settings and features will be set.

        Returns:
            Reports of any issues that were handled, but caused the state to not be exactly restored.
        """
        request = dto.SetStateRequest(
            interface_id=self.connection.interface_id,
            device=self.device_address,
            state=state,
            device_only=device_only,
        )
        response = await call_async(
            "device/set_device_state",
            request,
            SetStateDeviceResponse.from_binary)
        return response

    def can_set_state(
            self,
            state: str,
            firmware_version: Optional[FirmwareVersion] = None
    ) -> CanSetStateDeviceResponse:
        """
        Checks if a state can be applied to this device.
        This only covers exceptions that can be determined statically such as mismatches of ID or version,
        the process of applying the state can still fail when running.

        Args:
            state: The state object to check against.
            firmware_version: The firmware version of the device to apply the state to.
                Use this to ensure the state will still be compatible after an update.

        Returns:
            An object listing errors that come up when trying to set the state.
        """
        request = dto.CanSetStateRequest(
            interface_id=self.connection.interface_id,
            device=self.device_address,
            state=state,
            firmware_version=firmware_version,
        )
        response = call(
            "device/can_set_state",
            request,
            CanSetStateDeviceResponse.from_binary)
        return response

    async def can_set_state_async(
            self,
            state: str,
            firmware_version: Optional[FirmwareVersion] = None
    ) -> CanSetStateDeviceResponse:
        """
        Checks if a state can be applied to this device.
        This only covers exceptions that can be determined statically such as mismatches of ID or version,
        the process of applying the state can still fail when running.

        Args:
            state: The state object to check against.
            firmware_version: The firmware version of the device to apply the state to.
                Use this to ensure the state will still be compatible after an update.

        Returns:
            An object listing errors that come up when trying to set the state.
        """
        request = dto.CanSetStateRequest(
            interface_id=self.connection.interface_id,
            device=self.device_address,
            state=state,
            firmware_version=firmware_version,
        )
        response = await call_async(
            "device/can_set_state",
            request,
            CanSetStateDeviceResponse.from_binary)
        return response

    def wait_to_respond(
            self,
            timeout: float
    ) -> None:
        """
        Waits for the device to start responding to messages.
        Useful to call after resetting the device.
        Throws RequestTimeoutException upon timeout.

        Args:
            timeout: For how long to wait in milliseconds for the device to start responding.
        """
        request = dto.WaitToRespondRequest(
            interface_id=self.connection.interface_id,
            device=self.device_address,
            timeout=timeout,
        )
        call("device/wait_to_respond", request)

    async def wait_to_respond_async(
            self,
            timeout: float
    ) -> None:
        """
        Waits for the device to start responding to messages.
        Useful to call after resetting the device.
        Throws RequestTimeoutException upon timeout.

        Args:
            timeout: For how long to wait in milliseconds for the device to start responding.
        """
        request = dto.WaitToRespondRequest(
            interface_id=self.connection.interface_id,
            device=self.device_address,
            timeout=timeout,
        )
        await call_async("device/wait_to_respond", request)

    def renumber(
            self,
            address: int
    ) -> 'Device':
        """
        Changes the address of this device.
        After the address is successfully changed, the existing device class instance no longer represents the device.
        Instead, use the new device instance returned by this method.

        Args:
            address: The new address to assign to the device.

        Returns:
            New device instance with the new address.
        """
        if address < 1 or address > 99:
            raise ValueError('Invalid value; device addresses are numbered from 1 to 99.')

        request = dto.RenumberRequest(
            interface_id=self.connection.interface_id,
            device=self.device_address,
            address=address,
        )
        response = call(
            "device/renumber",
            request,
            dto.IntResponse.from_binary)
        return Device(self.connection, response.value)

    async def renumber_async(
            self,
            address: int
    ) -> 'Device':
        """
        Changes the address of this device.
        After the address is successfully changed, the existing device class instance no longer represents the device.
        Instead, use the new device instance returned by this method.

        Args:
            address: The new address to assign to the device.

        Returns:
            New device instance with the new address.
        """
        if address < 1 or address > 99:
            raise ValueError('Invalid value; device addresses are numbered from 1 to 99.')

        request = dto.RenumberRequest(
            interface_id=self.connection.interface_id,
            device=self.device_address,
            address=address,
        )
        response = await call_async(
            "device/renumber",
            request,
            dto.IntResponse.from_binary)
        return Device(self.connection, response.value)

    def restore(
            self,
            hard: bool = False
    ) -> None:
        """
        Restores most of the settings to their default values.
        Deletes all triggers, stream and PVT buffers, servo tunings.
        Deletes all zaber storage keys.
        Disables locksteps, unparks axes.
        Preserves storage, communication settings, peripherals (unless hard is specified).
        The device needs to be identified again after the restore.

        Args:
            hard: If true, completely erases device's memory. The device also resets.
        """
        request = dto.DeviceRestoreRequest(
            interface_id=self.connection.interface_id,
            device=self.device_address,
            hard=hard,
        )
        call("device/restore", request)

    async def restore_async(
            self,
            hard: bool = False
    ) -> None:
        """
        Restores most of the settings to their default values.
        Deletes all triggers, stream and PVT buffers, servo tunings.
        Deletes all zaber storage keys.
        Disables locksteps, unparks axes.
        Preserves storage, communication settings, peripherals (unless hard is specified).
        The device needs to be identified again after the restore.

        Args:
            hard: If true, completely erases device's memory. The device also resets.
        """
        request = dto.DeviceRestoreRequest(
            interface_id=self.connection.interface_id,
            device=self.device_address,
            hard=hard,
        )
        await call_async("device/restore", request)

    def __retrieve_identity(
            self
    ) -> DeviceIdentity:
        """
        Returns identity.

        Returns:
            Device identity.
        """
        request = dto.DeviceEmptyRequest(
            interface_id=self.connection.interface_id,
            device=self.device_address,
        )
        response = call_sync(
            "device/get_identity",
            request,
            DeviceIdentity.from_binary)
        return response

    def __retrieve_is_identified(
            self
    ) -> bool:
        """
        Returns whether or not the device have been identified.

        Returns:
            True if the device has already been identified. False otherwise.
        """
        request = dto.DeviceEmptyRequest(
            interface_id=self.connection.interface_id,
            device=self.device_address,
        )
        response = call_sync(
            "device/get_is_identified",
            request,
            dto.BoolResponse.from_binary)
        return response.value



# This file is generated. Do not modify by hand.
# pylint: disable=line-too-long, unused-argument, f-string-without-interpolation, too-many-branches, too-many-statements, unnecessary-pass
from dataclasses import dataclass
from typing import Any, Dict
import decimal
import zaber_bson
from ..ascii.trigger_action import TriggerAction
from ..ascii.trigger_operation import TriggerOperation


@dataclass
class TriggerOnFireSetToSettingRequest:

    interface_id: int = 0

    device: int = 0

    trigger_number: int = 0

    action: TriggerAction = next(first for first in TriggerAction)

    axis: int = 0

    setting: str = ""

    operation: TriggerOperation = next(first for first in TriggerOperation)

    from_axis: int = 0

    from_setting: str = ""

    @staticmethod
    def zero_values() -> 'TriggerOnFireSetToSettingRequest':
        return TriggerOnFireSetToSettingRequest(
            interface_id=0,
            device=0,
            trigger_number=0,
            action=next(first for first in TriggerAction),
            axis=0,
            setting="",
            operation=next(first for first in TriggerOperation),
            from_axis=0,
            from_setting="",
        )

    @staticmethod
    def from_binary(data_bytes: bytes) -> 'TriggerOnFireSetToSettingRequest':
        """" Deserialize a binary representation of this class. """
        data = zaber_bson.loads(data_bytes)  # type: Dict[str, Any]
        return TriggerOnFireSetToSettingRequest.from_dict(data)

    def to_binary(self) -> bytes:
        """" Serialize this class to a binary representation. """
        self.validate()
        return zaber_bson.dumps(self.to_dict())  # type: ignore

    def to_dict(self) -> Dict[str, Any]:
        return {
            'interfaceId': int(self.interface_id),
            'device': int(self.device),
            'triggerNumber': int(self.trigger_number),
            'action': self.action.value,
            'axis': int(self.axis),
            'setting': str(self.setting or ''),
            'operation': self.operation.value,
            'fromAxis': int(self.from_axis),
            'fromSetting': str(self.from_setting or ''),
        }

    @staticmethod
    def from_dict(data: Dict[str, Any]) -> 'TriggerOnFireSetToSettingRequest':
        return TriggerOnFireSetToSettingRequest(
            interface_id=data.get('interfaceId'),  # type: ignore
            device=data.get('device'),  # type: ignore
            trigger_number=data.get('triggerNumber'),  # type: ignore
            action=TriggerAction(data.get('action')),  # type: ignore
            axis=data.get('axis'),  # type: ignore
            setting=data.get('setting'),  # type: ignore
            operation=TriggerOperation(data.get('operation')),  # type: ignore
            from_axis=data.get('fromAxis'),  # type: ignore
            from_setting=data.get('fromSetting'),  # type: ignore
        )

    def validate(self) -> None:
        """" Validates the properties of the instance. """
        if self.interface_id is None:
            raise ValueError(f'Property "InterfaceId" of "TriggerOnFireSetToSettingRequest" is None.')

        if not isinstance(self.interface_id, (int, float, decimal.Decimal)):
            raise ValueError(f'Property "InterfaceId" of "TriggerOnFireSetToSettingRequest" is not a number.')

        if int(self.interface_id) != self.interface_id:
            raise ValueError(f'Property "InterfaceId" of "TriggerOnFireSetToSettingRequest" is not integer value.')

        if self.device is None:
            raise ValueError(f'Property "Device" of "TriggerOnFireSetToSettingRequest" is None.')

        if not isinstance(self.device, (int, float, decimal.Decimal)):
            raise ValueError(f'Property "Device" of "TriggerOnFireSetToSettingRequest" is not a number.')

        if int(self.device) != self.device:
            raise ValueError(f'Property "Device" of "TriggerOnFireSetToSettingRequest" is not integer value.')

        if self.trigger_number is None:
            raise ValueError(f'Property "TriggerNumber" of "TriggerOnFireSetToSettingRequest" is None.')

        if not isinstance(self.trigger_number, (int, float, decimal.Decimal)):
            raise ValueError(f'Property "TriggerNumber" of "TriggerOnFireSetToSettingRequest" is not a number.')

        if int(self.trigger_number) != self.trigger_number:
            raise ValueError(f'Property "TriggerNumber" of "TriggerOnFireSetToSettingRequest" is not integer value.')

        if self.action is None:
            raise ValueError(f'Property "Action" of "TriggerOnFireSetToSettingRequest" is None.')

        if not isinstance(self.action, TriggerAction):
            raise ValueError(f'Property "Action" of "TriggerOnFireSetToSettingRequest" is not an instance of "TriggerAction".')

        if self.axis is None:
            raise ValueError(f'Property "Axis" of "TriggerOnFireSetToSettingRequest" is None.')

        if not isinstance(self.axis, (int, float, decimal.Decimal)):
            raise ValueError(f'Property "Axis" of "TriggerOnFireSetToSettingRequest" is not a number.')

        if int(self.axis) != self.axis:
            raise ValueError(f'Property "Axis" of "TriggerOnFireSetToSettingRequest" is not integer value.')

        if self.setting is not None:
            if not isinstance(self.setting, str):
                raise ValueError(f'Property "Setting" of "TriggerOnFireSetToSettingRequest" is not a string.')

        if self.operation is None:
            raise ValueError(f'Property "Operation" of "TriggerOnFireSetToSettingRequest" is None.')

        if not isinstance(self.operation, TriggerOperation):
            raise ValueError(f'Property "Operation" of "TriggerOnFireSetToSettingRequest" is not an instance of "TriggerOperation".')

        if self.from_axis is None:
            raise ValueError(f'Property "FromAxis" of "TriggerOnFireSetToSettingRequest" is None.')

        if not isinstance(self.from_axis, (int, float, decimal.Decimal)):
            raise ValueError(f'Property "FromAxis" of "TriggerOnFireSetToSettingRequest" is not a number.')

        if int(self.from_axis) != self.from_axis:
            raise ValueError(f'Property "FromAxis" of "TriggerOnFireSetToSettingRequest" is not integer value.')

        if self.from_setting is not None:
            if not isinstance(self.from_setting, str):
                raise ValueError(f'Property "FromSetting" of "TriggerOnFireSetToSettingRequest" is not a string.')



# This file is generated. Do not modify by hand.
# pylint: disable=line-too-long, unused-argument, f-string-without-interpolation, too-many-branches, too-many-statements, unnecessary-pass
from dataclasses import dataclass
from typing import Any, Dict
import decimal
import zaber_bson
from ...units import Units, UnitsAndLiterals, units_from_literals


@dataclass
class DeviceMoveSinRequest:

    interface_id: int = 0

    device: int = 0

    axis: int = 0

    amplitude: float = 0

    amplitude_units: UnitsAndLiterals = Units.NATIVE

    period: float = 0

    period_units: UnitsAndLiterals = Units.NATIVE

    count: float = 0

    wait_until_idle: bool = False

    @staticmethod
    def zero_values() -> 'DeviceMoveSinRequest':
        return DeviceMoveSinRequest(
            interface_id=0,
            device=0,
            axis=0,
            amplitude=0,
            amplitude_units=Units.NATIVE,
            period=0,
            period_units=Units.NATIVE,
            count=0,
            wait_until_idle=False,
        )

    @staticmethod
    def from_binary(data_bytes: bytes) -> 'DeviceMoveSinRequest':
        """" Deserialize a binary representation of this class. """
        data = zaber_bson.loads(data_bytes)  # type: Dict[str, Any]
        return DeviceMoveSinRequest.from_dict(data)

    def to_binary(self) -> bytes:
        """" Serialize this class to a binary representation. """
        self.validate()
        return zaber_bson.dumps(self.to_dict())  # type: ignore

    def to_dict(self) -> Dict[str, Any]:
        return {
            'interfaceId': int(self.interface_id),
            'device': int(self.device),
            'axis': int(self.axis),
            'amplitude': float(self.amplitude),
            'amplitudeUnits': units_from_literals(self.amplitude_units).value,
            'period': float(self.period),
            'periodUnits': units_from_literals(self.period_units).value,
            'count': float(self.count),
            'waitUntilIdle': bool(self.wait_until_idle),
        }

    @staticmethod
    def from_dict(data: Dict[str, Any]) -> 'DeviceMoveSinRequest':
        return DeviceMoveSinRequest(
            interface_id=data.get('interfaceId'),  # type: ignore
            device=data.get('device'),  # type: ignore
            axis=data.get('axis'),  # type: ignore
            amplitude=data.get('amplitude'),  # type: ignore
            amplitude_units=Units(data.get('amplitudeUnits')),  # type: ignore
            period=data.get('period'),  # type: ignore
            period_units=Units(data.get('periodUnits')),  # type: ignore
            count=data.get('count'),  # type: ignore
            wait_until_idle=data.get('waitUntilIdle'),  # type: ignore
        )

    def validate(self) -> None:
        """" Validates the properties of the instance. """
        if self.interface_id is None:
            raise ValueError(f'Property "InterfaceId" of "DeviceMoveSinRequest" is None.')

        if not isinstance(self.interface_id, (int, float, decimal.Decimal)):
            raise ValueError(f'Property "InterfaceId" of "DeviceMoveSinRequest" is not a number.')

        if int(self.interface_id) != self.interface_id:
            raise ValueError(f'Property "InterfaceId" of "DeviceMoveSinRequest" is not integer value.')

        if self.device is None:
            raise ValueError(f'Property "Device" of "DeviceMoveSinRequest" is None.')

        if not isinstance(self.device, (int, float, decimal.Decimal)):
            raise ValueError(f'Property "Device" of "DeviceMoveSinRequest" is not a number.')

        if int(self.device) != self.device:
            raise ValueError(f'Property "Device" of "DeviceMoveSinRequest" is not integer value.')

        if self.axis is None:
            raise ValueError(f'Property "Axis" of "DeviceMoveSinRequest" is None.')

        if not isinstance(self.axis, (int, float, decimal.Decimal)):
            raise ValueError(f'Property "Axis" of "DeviceMoveSinRequest" is not a number.')

        if int(self.axis) != self.axis:
            raise ValueError(f'Property "Axis" of "DeviceMoveSinRequest" is not integer value.')

        if self.amplitude is None:
            raise ValueError(f'Property "Amplitude" of "DeviceMoveSinRequest" is None.')

        if not isinstance(self.amplitude, (int, float, decimal.Decimal)):
            raise ValueError(f'Property "Amplitude" of "DeviceMoveSinRequest" is not a number.')

        if self.amplitude_units is None:
            raise ValueError(f'Property "AmplitudeUnits" of "DeviceMoveSinRequest" is None.')

        if not isinstance(self.amplitude_units, (Units, str)):
            raise ValueError(f'Property "AmplitudeUnits" of "DeviceMoveSinRequest" is not Units.')

        if self.period is None:
            raise ValueError(f'Property "Period" of "DeviceMoveSinRequest" is None.')

        if not isinstance(self.period, (int, float, decimal.Decimal)):
            raise ValueError(f'Property "Period" of "DeviceMoveSinRequest" is not a number.')

        if self.period_units is None:
            raise ValueError(f'Property "PeriodUnits" of "DeviceMoveSinRequest" is None.')

        if not isinstance(self.period_units, (Units, str)):
            raise ValueError(f'Property "PeriodUnits" of "DeviceMoveSinRequest" is not Units.')

        if self.count is None:
            raise ValueError(f'Property "Count" of "DeviceMoveSinRequest" is None.')

        if not isinstance(self.count, (int, float, decimal.Decimal)):
            raise ValueError(f'Property "Count" of "DeviceMoveSinRequest" is not a number.')



# This file is generated. Do not modify by hand.
# pylint: disable=line-too-long, unused-argument, f-string-without-interpolation, too-many-branches, too-many-statements, unnecessary-pass
from dataclasses import dataclass
from typing import Any, Dict, List
import decimal
from collections.abc import Iterable
import zaber_bson


@dataclass
class DeviceAddressConflictExceptionData:
    """
    Contains additional data for DeviceAddressConflictException.
    """

    device_addresses: List[int]
    """
    The full list of detected device addresses.
    """

    @staticmethod
    def zero_values() -> 'DeviceAddressConflictExceptionData':
        return DeviceAddressConflictExceptionData(
            device_addresses=[],
        )

    @staticmethod
    def from_binary(data_bytes: bytes) -> 'DeviceAddressConflictExceptionData':
        """" Deserialize a binary representation of this class. """
        data = zaber_bson.loads(data_bytes)  # type: Dict[str, Any]
        return DeviceAddressConflictExceptionData.from_dict(data)

    def to_binary(self) -> bytes:
        """" Serialize this class to a binary representation. """
        self.validate()
        return zaber_bson.dumps(self.to_dict())  # type: ignore

    def to_dict(self) -> Dict[str, Any]:
        return {
            'deviceAddresses': [int(item) for item in self.device_addresses] if self.device_addresses is not None else [],
        }

    @staticmethod
    def from_dict(data: Dict[str, Any]) -> 'DeviceAddressConflictExceptionData':
        return DeviceAddressConflictExceptionData(
            device_addresses=data.get('deviceAddresses'),  # type: ignore
        )

    def validate(self) -> None:
        """" Validates the properties of the instance. """
        if self.device_addresses is not None:
            if not isinstance(self.device_addresses, Iterable):
                raise ValueError('Property "DeviceAddresses" of "DeviceAddressConflictExceptionData" is not iterable.')

            for i, device_addresses_item in enumerate(self.device_addresses):
                if device_addresses_item is None:
                    raise ValueError(f'Item {i} in property "DeviceAddresses" of "DeviceAddressConflictExceptionData" is None.')

                if not isinstance(device_addresses_item, (int, float, decimal.Decimal)):
                    raise ValueError(f'Item {i} in property "DeviceAddresses" of "DeviceAddressConflictExceptionData" is not a number.')

                if int(device_addresses_item) != device_addresses_item:
                    raise ValueError(f'Item {i} in property "DeviceAddresses" of "DeviceAddressConflictExceptionData" is not integer value.')



# ===== THIS FILE IS GENERATED FROM A TEMPLATE ===== #
# ============== DO NOT EDIT DIRECTLY ============== #

from typing import Union
from ..dto.exceptions.device_address_conflict_exception_data import DeviceAddressConflictExceptionData
from .motion_lib_exception import MotionLibException


class DeviceAddressConflictException(MotionLibException):
    """
    Thrown when there is a conflict in device numbers preventing unique addressing.
    """

    @property
    def details(self) -> DeviceAddressConflictExceptionData:
        """
        Additional data for DeviceAddressConflictException
        """
        return self._details

    def __init__(self, message: str, custom_data: Union[bytes, DeviceAddressConflictExceptionData]):
        MotionLibException.__init__(self, message)

        if isinstance(custom_data, DeviceAddressConflictExceptionData):
            self._details = custom_data
        else:
            self._details = DeviceAddressConflictExceptionData.from_binary(custom_data)



# This file is generated. Do not modify by hand.
# pylint: disable=line-too-long, unused-argument, f-string-without-interpolation, too-many-branches, too-many-statements, unnecessary-pass
from dataclasses import dataclass
from typing import Any, Dict, List, Optional
import decimal
from collections.abc import Iterable
import zaber_bson
from .servo_tuning_param import ServoTuningParam


@dataclass
class SimpleTuning:
    """
    The masses and parameters last used by simple tuning.
    """

    is_used: bool
    """
    Whether the tuning returned is currently in use by this paramset,
    or if it has been overwritten by a later change.
    """

    is_set: bool
    """
    If this paramset has been tuned using the simple tuning method, whether or not it's currently in use.
    """

    load_mass: float
    """
    The mass of the load in kg, excluding the mass of the carriage.
    """

    tuning_params: List[ServoTuningParam]
    """
    The parameters used by simple tuning.
    """

    carriage_mass: Optional[float] = None
    """
    The mass of the carriage in kg.
    """

    motor_inertia: Optional[float] = None
    """
    The mass of the carriage in kg.
    """

    @staticmethod
    def zero_values() -> 'SimpleTuning':
        return SimpleTuning(
            is_used=False,
            is_set=False,
            carriage_mass=None,
            motor_inertia=None,
            load_mass=0,
            tuning_params=[],
        )

    @staticmethod
    def from_binary(data_bytes: bytes) -> 'SimpleTuning':
        """" Deserialize a binary representation of this class. """
        data = zaber_bson.loads(data_bytes)  # type: Dict[str, Any]
        return SimpleTuning.from_dict(data)

    def to_binary(self) -> bytes:
        """" Serialize this class to a binary representation. """
        self.validate()
        return zaber_bson.dumps(self.to_dict())  # type: ignore

    def to_dict(self) -> Dict[str, Any]:
        return {
            'isUsed': bool(self.is_used),
            'isSet': bool(self.is_set),
            'carriageMass': float(self.carriage_mass) if self.carriage_mass is not None else None,
            'motorInertia': float(self.motor_inertia) if self.motor_inertia is not None else None,
            'loadMass': float(self.load_mass),
            'tuningParams': [item.to_dict() for item in self.tuning_params] if self.tuning_params is not None else [],
        }

    @staticmethod
    def from_dict(data: Dict[str, Any]) -> 'SimpleTuning':
        return SimpleTuning(
            is_used=data.get('isUsed'),  # type: ignore
            is_set=data.get('isSet'),  # type: ignore
            carriage_mass=data.get('carriageMass'),  # type: ignore
            motor_inertia=data.get('motorInertia'),  # type: ignore
            load_mass=data.get('loadMass'),  # type: ignore
            tuning_params=[ServoTuningParam.from_dict(item) for item in data.get('tuningParams')],  # type: ignore
        )

    def validate(self) -> None:
        """" Validates the properties of the instance. """
        if self.carriage_mass is not None:
            if not isinstance(self.carriage_mass, (int, float, decimal.Decimal)):
                raise ValueError(f'Property "CarriageMass" of "SimpleTuning" is not a number.')

        if self.motor_inertia is not None:
            if not isinstance(self.motor_inertia, (int, float, decimal.Decimal)):
                raise ValueError(f'Property "MotorInertia" of "SimpleTuning" is not a number.')

        if self.load_mass is None:
            raise ValueError(f'Property "LoadMass" of "SimpleTuning" is None.')

        if not isinstance(self.load_mass, (int, float, decimal.Decimal)):
            raise ValueError(f'Property "LoadMass" of "SimpleTuning" is not a number.')

        if self.tuning_params is not None:
            if not isinstance(self.tuning_params, Iterable):
                raise ValueError('Property "TuningParams" of "SimpleTuning" is not iterable.')

            for i, tuning_params_item in enumerate(self.tuning_params):
                if tuning_params_item is None:
                    raise ValueError(f'Item {i} in property "TuningParams" of "SimpleTuning" is None.')

                if not isinstance(tuning_params_item, ServoTuningParam):
                    raise ValueError(f'Item {i} in property "TuningParams" of "SimpleTuning" is not an instance of "ServoTuningParam".')

                tuning_params_item.validate()



# pylint: disable=too-many-arguments, too-many-lines

# ===== THIS FILE IS GENERATED FROM A TEMPLATE ===== #
# ============== DO NOT EDIT DIRECTLY ============== #
from typing import List, Optional
from ..dto import requests as dto
from ..call import call, call_async
from .axis import Axis
from ..dto.ascii.servo_tuning_paramset import ServoTuningParamset
from ..dto.ascii.paramset_info import ParamsetInfo
from ..dto.ascii.servo_tuning_param import ServoTuningParam
from ..dto.ascii.simple_tuning import SimpleTuning
from ..dto.ascii.simple_tuning_param_definition import SimpleTuningParamDefinition
from ..dto.ascii.pid_tuning import PidTuning
from ..units import UnitsAndLiterals, Units


class ServoTuner:
    """
    Exposes the capabilities to inspect and edit an axis' servo tuning.
    Requires at least Firmware 6.25 or 7.00.
    """

    @property
    def axis(self) -> Axis:
        """
        The axis that will be tuned.
        """
        return self._axis

    def __init__(self, axis: Axis):
        """
        Creates instance of ServoTuner for the given axis.
        """
        self._axis = axis

    def get_startup_paramset(
            self
    ) -> ServoTuningParamset:
        """
        Get the paramset that this device uses by default when it starts up.

        Returns:
            The paramset used when the device restarts.
        """
        request = dto.AxisEmptyRequest(
            interface_id=self.axis.device.connection.interface_id,
            device=self.axis.device.device_address,
            axis=self.axis.axis_number,
        )
        response = call(
            "servotuning/get_startup_set",
            request,
            dto.ServoTuningParamsetResponse.from_binary)
        return response.paramset

    async def get_startup_paramset_async(
            self
    ) -> ServoTuningParamset:
        """
        Get the paramset that this device uses by default when it starts up.

        Returns:
            The paramset used when the device restarts.
        """
        request = dto.AxisEmptyRequest(
            interface_id=self.axis.device.connection.interface_id,
            device=self.axis.device.device_address,
            axis=self.axis.axis_number,
        )
        response = await call_async(
            "servotuning/get_startup_set",
            request,
            dto.ServoTuningParamsetResponse.from_binary)
        return response.paramset

    def set_startup_paramset(
            self,
            paramset: ServoTuningParamset
    ) -> None:
        """
        Set the paramset that this device uses by default when it starts up.

        Args:
            paramset: The paramset to use at startup.
        """
        request = dto.ServoTuningRequest(
            interface_id=self.axis.device.connection.interface_id,
            device=self.axis.device.device_address,
            axis=self.axis.axis_number,
            paramset=paramset,
        )
        call("servotuning/set_startup_set", request)

    async def set_startup_paramset_async(
            self,
            paramset: ServoTuningParamset
    ) -> None:
        """
        Set the paramset that this device uses by default when it starts up.

        Args:
            paramset: The paramset to use at startup.
        """
        request = dto.ServoTuningRequest(
            interface_id=self.axis.device.connection.interface_id,
            device=self.axis.device.device_address,
            axis=self.axis.axis_number,
            paramset=paramset,
        )
        await call_async("servotuning/set_startup_set", request)

    def load_paramset(
            self,
            to_paramset: ServoTuningParamset,
            from_paramset: ServoTuningParamset
    ) -> None:
        """
        Load the values from one paramset into another.

        Args:
            to_paramset: The paramset to load into.
            from_paramset: The paramset to load from.
        """
        request = dto.LoadParamset(
            interface_id=self.axis.device.connection.interface_id,
            device=self.axis.device.device_address,
            axis=self.axis.axis_number,
            to_paramset=to_paramset,
            from_paramset=from_paramset,
        )
        call("servotuning/load_paramset", request)

    async def load_paramset_async(
            self,
            to_paramset: ServoTuningParamset,
            from_paramset: ServoTuningParamset
    ) -> None:
        """
        Load the values from one paramset into another.

        Args:
            to_paramset: The paramset to load into.
            from_paramset: The paramset to load from.
        """
        request = dto.LoadParamset(
            interface_id=self.axis.device.connection.interface_id,
            device=self.axis.device.device_address,
            axis=self.axis.axis_number,
            to_paramset=to_paramset,
            from_paramset=from_paramset,
        )
        await call_async("servotuning/load_paramset", request)

    def get_tuning(
            self,
            paramset: ServoTuningParamset
    ) -> ParamsetInfo:
        """
        Get the full set of tuning parameters used by the firmware driving this axis.

        Args:
            paramset: The paramset to get tuning for.

        Returns:
            The raw representation of the current tuning.
        """
        request = dto.ServoTuningRequest(
            interface_id=self.axis.device.connection.interface_id,
            device=self.axis.device.device_address,
            axis=self.axis.axis_number,
            paramset=paramset,
        )
        response = call(
            "servotuning/get_raw",
            request,
            ParamsetInfo.from_binary)
        return response

    async def get_tuning_async(
            self,
            paramset: ServoTuningParamset
    ) -> ParamsetInfo:
        """
        Get the full set of tuning parameters used by the firmware driving this axis.

        Args:
            paramset: The paramset to get tuning for.

        Returns:
            The raw representation of the current tuning.
        """
        request = dto.ServoTuningRequest(
            interface_id=self.axis.device.connection.interface_id,
            device=self.axis.device.device_address,
            axis=self.axis.axis_number,
            paramset=paramset,
        )
        response = await call_async(
            "servotuning/get_raw",
            request,
            ParamsetInfo.from_binary)
        return response

    def set_tuning(
            self,
            paramset: ServoTuningParamset,
            tuning_params: List[ServoTuningParam],
            set_unspecified_to_default: bool = False
    ) -> None:
        """
        Set individual tuning parameters.
        Only use this method if you have a strong understanding of Zaber specific tuning parameters.

        Args:
            paramset: The paramset to set tuning of.
            tuning_params: The params to set.
            set_unspecified_to_default: If true, any tuning parameters not included in TuningParams
                are reset to their default values.
        """
        request = dto.SetServoTuningRequest(
            interface_id=self.axis.device.connection.interface_id,
            device=self.axis.device.device_address,
            axis=self.axis.axis_number,
            paramset=paramset,
            tuning_params=tuning_params,
            set_unspecified_to_default=set_unspecified_to_default,
        )
        call("servotuning/set_raw", request)

    async def set_tuning_async(
            self,
            paramset: ServoTuningParamset,
            tuning_params: List[ServoTuningParam],
            set_unspecified_to_default: bool = False
    ) -> None:
        """
        Set individual tuning parameters.
        Only use this method if you have a strong understanding of Zaber specific tuning parameters.

        Args:
            paramset: The paramset to set tuning of.
            tuning_params: The params to set.
            set_unspecified_to_default: If true, any tuning parameters not included in TuningParams
                are reset to their default values.
        """
        request = dto.SetServoTuningRequest(
            interface_id=self.axis.device.connection.interface_id,
            device=self.axis.device.device_address,
            axis=self.axis.axis_number,
            paramset=paramset,
            tuning_params=tuning_params,
            set_unspecified_to_default=set_unspecified_to_default,
        )
        await call_async("servotuning/set_raw", request)

    def set_pid_tuning(
            self,
            paramset: ServoTuningParamset,
            p: float,
            i: float,
            d: float,
            fc: float
    ) -> PidTuning:
        """
        Sets the tuning of a paramset using the PID method.

        Args:
            paramset: The paramset to get tuning for.
            p: The proportional gain. Must be in units of N/m for linear devices, and N⋅m/° for rotary devices.
            i: The integral gain. Must be in units of N/(m⋅s) for linear devices, and N⋅m/(°⋅s) for rotary devices.
            d: The derivative gain. Must be in units of N⋅s/m for linear devices, and N⋅m⋅s/° for rotary devices.
            fc: The cutoff frequency. Must be in units of Hz.

        Returns:
            The PID representation of the current tuning after your changes have been applied.
        """
        request = dto.SetServoTuningPIDRequest(
            interface_id=self.axis.device.connection.interface_id,
            device=self.axis.device.device_address,
            axis=self.axis.axis_number,
            paramset=paramset,
            p=p,
            i=i,
            d=d,
            fc=fc,
        )
        response = call(
            "servotuning/set_pid",
            request,
            PidTuning.from_binary)
        return response

    async def set_pid_tuning_async(
            self,
            paramset: ServoTuningParamset,
            p: float,
            i: float,
            d: float,
            fc: float
    ) -> PidTuning:
        """
        Sets the tuning of a paramset using the PID method.

        Args:
            paramset: The paramset to get tuning for.
            p: The proportional gain. Must be in units of N/m for linear devices, and N⋅m/° for rotary devices.
            i: The integral gain. Must be in units of N/(m⋅s) for linear devices, and N⋅m/(°⋅s) for rotary devices.
            d: The derivative gain. Must be in units of N⋅s/m for linear devices, and N⋅m⋅s/° for rotary devices.
            fc: The cutoff frequency. Must be in units of Hz.

        Returns:
            The PID representation of the current tuning after your changes have been applied.
        """
        request = dto.SetServoTuningPIDRequest(
            interface_id=self.axis.device.connection.interface_id,
            device=self.axis.device.device_address,
            axis=self.axis.axis_number,
            paramset=paramset,
            p=p,
            i=i,
            d=d,
            fc=fc,
        )
        response = await call_async(
            "servotuning/set_pid",
            request,
            PidTuning.from_binary)
        return response

    def get_pid_tuning(
            self,
            paramset: ServoTuningParamset
    ) -> PidTuning:
        """
        Gets the PID representation of this paramset's servo tuning.

        Args:
            paramset: The paramset to get tuning for.

        Returns:
            The PID representation of the current tuning.
        """
        request = dto.ServoTuningRequest(
            interface_id=self.axis.device.connection.interface_id,
            device=self.axis.device.device_address,
            axis=self.axis.axis_number,
            paramset=paramset,
        )
        response = call(
            "servotuning/get_pid",
            request,
            PidTuning.from_binary)
        return response

    async def get_pid_tuning_async(
            self,
            paramset: ServoTuningParamset
    ) -> PidTuning:
        """
        Gets the PID representation of this paramset's servo tuning.

        Args:
            paramset: The paramset to get tuning for.

        Returns:
            The PID representation of the current tuning.
        """
        request = dto.ServoTuningRequest(
            interface_id=self.axis.device.connection.interface_id,
            device=self.axis.device.device_address,
            axis=self.axis.axis_number,
            paramset=paramset,
        )
        response = await call_async(
            "servotuning/get_pid",
            request,
            PidTuning.from_binary)
        return response

    def get_simple_tuning_param_definitions(
            self
    ) -> List[SimpleTuningParamDefinition]:
        """
        Gets the parameters that are required to tune this device.

        Returns:
            The tuning parameters.
        """
        request = dto.AxisEmptyRequest(
            interface_id=self.axis.device.connection.interface_id,
            device=self.axis.device.device_address,
            axis=self.axis.axis_number,
        )
        response = call(
            "servotuning/get_simple_params_definition",
            request,
            dto.GetSimpleTuningParamDefinitionResponse.from_binary)
        return response.params

    async def get_simple_tuning_param_definitions_async(
            self
    ) -> List[SimpleTuningParamDefinition]:
        """
        Gets the parameters that are required to tune this device.

        Returns:
            The tuning parameters.
        """
        request = dto.AxisEmptyRequest(
            interface_id=self.axis.device.connection.interface_id,
            device=self.axis.device.device_address,
            axis=self.axis.axis_number,
        )
        response = await call_async(
            "servotuning/get_simple_params_definition",
            request,
            dto.GetSimpleTuningParamDefinitionResponse.from_binary)
        return response.params

    def set_simple_tuning(
            self,
            paramset: ServoTuningParamset,
            tuning_params: List[ServoTuningParam],
            load_mass: float,
            load_mass_units: UnitsAndLiterals = Units.NATIVE,
            carriage_mass: Optional[float] = None,
            carriage_mass_units: UnitsAndLiterals = Units.NATIVE,
            motor_inertia: Optional[float] = None,
            motor_inertia_units: UnitsAndLiterals = Units.NATIVE
    ) -> None:
        """
        Set the tuning of this device using the simple input method.

        Args:
            paramset: The paramset to set tuning for.
            tuning_params: The params used to tune this device.
                To get what parameters are expected, call GetSimpleTuningParamList.
                All values must be between 0 and 1.
            load_mass: The mass loaded on the stage, excluding the mass of the carriage itself.
                Unless specified by the LoadMassUnits parameter, this is in units of kg for linear devices,
                and kg⋅m² for rotary devices.
            load_mass_units: The units the load mass was supplied in.
            carriage_mass: The mass of the carriage itself. If not supplied, the product's default mass will be used.
                Unless specified by the CarriageMassUnits parameter, this is in units of kg for linear devices,
                and kg⋅m² for rotary devices.
            carriage_mass_units: The units the carriage mass was supplied in.
            motor_inertia: The inertia of the motor. Unless specified by the MotorInertiaUnits parameter,
                this is in units of kg⋅m².
            motor_inertia_units: The units the motor inertia was supplied in.
        """
        request = dto.SetSimpleTuning(
            interface_id=self.axis.device.connection.interface_id,
            device=self.axis.device.device_address,
            axis=self.axis.axis_number,
            paramset=paramset,
            tuning_params=tuning_params,
            load_mass=load_mass,
            load_mass_units=load_mass_units,
            carriage_mass=carriage_mass,
            carriage_mass_units=carriage_mass_units,
            motor_inertia=motor_inertia,
            motor_inertia_units=motor_inertia_units,
        )
        call("servotuning/set_simple_tuning", request)

    async def set_simple_tuning_async(
            self,
            paramset: ServoTuningParamset,
            tuning_params: List[ServoTuningParam],
            load_mass: float,
            load_mass_units: UnitsAndLiterals = Units.NATIVE,
            carriage_mass: Optional[float] = None,
            carriage_mass_units: UnitsAndLiterals = Units.NATIVE,
            motor_inertia: Optional[float] = None,
            motor_inertia_units: UnitsAndLiterals = Units.NATIVE
    ) -> None:
        """
        Set the tuning of this device using the simple input method.

        Args:
            paramset: The paramset to set tuning for.
            tuning_params: The params used to tune this device.
                To get what parameters are expected, call GetSimpleTuningParamList.
                All values must be between 0 and 1.
            load_mass: The mass loaded on the stage, excluding the mass of the carriage itself.
                Unless specified by the LoadMassUnits parameter, this is in units of kg for linear devices,
                and kg⋅m² for rotary devices.
            load_mass_units: The units the load mass was supplied in.
            carriage_mass: The mass of the carriage itself. If not supplied, the product's default mass will be used.
                Unless specified by the CarriageMassUnits parameter, this is in units of kg for linear devices,
                and kg⋅m² for rotary devices.
            carriage_mass_units: The units the carriage mass was supplied in.
            motor_inertia: The inertia of the motor. Unless specified by the MotorInertiaUnits parameter,
                this is in units of kg⋅m².
            motor_inertia_units: The units the motor inertia was supplied in.
        """
        request = dto.SetSimpleTuning(
            interface_id=self.axis.device.connection.interface_id,
            device=self.axis.device.device_address,
            axis=self.axis.axis_number,
            paramset=paramset,
            tuning_params=tuning_params,
            load_mass=load_mass,
            load_mass_units=load_mass_units,
            carriage_mass=carriage_mass,
            carriage_mass_units=carriage_mass_units,
            motor_inertia=motor_inertia,
            motor_inertia_units=motor_inertia_units,
        )
        await call_async("servotuning/set_simple_tuning", request)

    def get_simple_tuning(
            self,
            paramset: ServoTuningParamset
    ) -> SimpleTuning:
        """
        Get the simple tuning parameters for this device.

        Args:
            paramset: The paramset to get tuning for.

        Returns:
            The simple tuning parameters.
        """
        request = dto.ServoTuningRequest(
            interface_id=self.axis.device.connection.interface_id,
            device=self.axis.device.device_address,
            axis=self.axis.axis_number,
            paramset=paramset,
        )
        response = call(
            "servotuning/get_simple_tuning",
            request,
            SimpleTuning.from_binary)
        return response

    async def get_simple_tuning_async(
            self,
            paramset: ServoTuningParamset
    ) -> SimpleTuning:
        """
        Get the simple tuning parameters for this device.

        Args:
            paramset: The paramset to get tuning for.

        Returns:
            The simple tuning parameters.
        """
        request = dto.ServoTuningRequest(
            interface_id=self.axis.device.connection.interface_id,
            device=self.axis.device.device_address,
            axis=self.axis.axis_number,
            paramset=paramset,
        )
        response = await call_async(
            "servotuning/get_simple_tuning",
            request,
            SimpleTuning.from_binary)
        return response

    def is_using_simple_tuning(
            self,
            paramset: ServoTuningParamset,
            tuning_params: List[ServoTuningParam],
            load_mass: float,
            carriage_mass: Optional[float] = None
    ) -> bool:
        """
        Deprecated: Use GetSimpleTuning instead.

        Checks if the provided simple tuning is being stored by this paramset.

        Args:
            paramset: The paramset to set tuning for.
            tuning_params: The params used to tune this device.
                To get what parameters are expected, call GetSimpleTuningParamList.
                All values must be between 0 and 1.
            load_mass: The mass loaded on the stage (excluding the mass of the carriage itself) in kg.
            carriage_mass: The mass of the carriage in kg. If this value is not set the default carriage mass is used.

        Returns:
            True if the provided simple tuning is currently stored in this paramset.
        """
        request = dto.SetSimpleTuning(
            interface_id=self.axis.device.connection.interface_id,
            device=self.axis.device.device_address,
            axis=self.axis.axis_number,
            paramset=paramset,
            tuning_params=tuning_params,
            load_mass=load_mass,
            carriage_mass=carriage_mass,
        )
        response = call(
            "servotuning/is_using_simple_tuning",
            request,
            dto.BoolResponse.from_binary)
        return response.value

    async def is_using_simple_tuning_async(
            self,
            paramset: ServoTuningParamset,
            tuning_params: List[ServoTuningParam],
            load_mass: float,
            carriage_mass: Optional[float] = None
    ) -> bool:
        """
        Deprecated: Use GetSimpleTuning instead.

        Checks if the provided simple tuning is being stored by this paramset.

        Args:
            paramset: The paramset to set tuning for.
            tuning_params: The params used to tune this device.
                To get what parameters are expected, call GetSimpleTuningParamList.
                All values must be between 0 and 1.
            load_mass: The mass loaded on the stage (excluding the mass of the carriage itself) in kg.
            carriage_mass: The mass of the carriage in kg. If this value is not set the default carriage mass is used.

        Returns:
            True if the provided simple tuning is currently stored in this paramset.
        """
        request = dto.SetSimpleTuning(
            interface_id=self.axis.device.connection.interface_id,
            device=self.axis.device.device_address,
            axis=self.axis.axis_number,
            paramset=paramset,
            tuning_params=tuning_params,
            load_mass=load_mass,
            carriage_mass=carriage_mass,
        )
        response = await call_async(
            "servotuning/is_using_simple_tuning",
            request,
            dto.BoolResponse.from_binary)
        return response.value



# This file is generated. Do not modify by hand.
# pylint: disable=line-too-long, unused-argument, f-string-without-interpolation, too-many-branches, too-many-statements, unnecessary-pass
from dataclasses import dataclass, field
from typing import Any, Dict, List, Optional
import decimal
from collections.abc import Iterable
import zaber_bson
from ..gcode.translator_config import TranslatorConfig


@dataclass
class TranslatorCreateFromDeviceRequest:

    interface_id: int = 0

    device: int = 0

    axes: List[int] = field(default_factory=list)

    config: Optional[TranslatorConfig] = None

    @staticmethod
    def zero_values() -> 'TranslatorCreateFromDeviceRequest':
        return TranslatorCreateFromDeviceRequest(
            interface_id=0,
            device=0,
            axes=[],
            config=None,
        )

    @staticmethod
    def from_binary(data_bytes: bytes) -> 'TranslatorCreateFromDeviceRequest':
        """" Deserialize a binary representation of this class. """
        data = zaber_bson.loads(data_bytes)  # type: Dict[str, Any]
        return TranslatorCreateFromDeviceRequest.from_dict(data)

    def to_binary(self) -> bytes:
        """" Serialize this class to a binary representation. """
        self.validate()
        return zaber_bson.dumps(self.to_dict())  # type: ignore

    def to_dict(self) -> Dict[str, Any]:
        return {
            'interfaceId': int(self.interface_id),
            'device': int(self.device),
            'axes': [int(item) for item in self.axes] if self.axes is not None else [],
            'config': self.config.to_dict() if self.config is not None else None,
        }

    @staticmethod
    def from_dict(data: Dict[str, Any]) -> 'TranslatorCreateFromDeviceRequest':
        return TranslatorCreateFromDeviceRequest(
            interface_id=data.get('interfaceId'),  # type: ignore
            device=data.get('device'),  # type: ignore
            axes=data.get('axes'),  # type: ignore
            config=TranslatorConfig.from_dict(data.get('config')) if data.get('config') is not None else None,  # type: ignore
        )

    def validate(self) -> None:
        """" Validates the properties of the instance. """
        if self.interface_id is None:
            raise ValueError(f'Property "InterfaceId" of "TranslatorCreateFromDeviceRequest" is None.')

        if not isinstance(self.interface_id, (int, float, decimal.Decimal)):
            raise ValueError(f'Property "InterfaceId" of "TranslatorCreateFromDeviceRequest" is not a number.')

        if int(self.interface_id) != self.interface_id:
            raise ValueError(f'Property "InterfaceId" of "TranslatorCreateFromDeviceRequest" is not integer value.')

        if self.device is None:
            raise ValueError(f'Property "Device" of "TranslatorCreateFromDeviceRequest" is None.')

        if not isinstance(self.device, (int, float, decimal.Decimal)):
            raise ValueError(f'Property "Device" of "TranslatorCreateFromDeviceRequest" is not a number.')

        if int(self.device) != self.device:
            raise ValueError(f'Property "Device" of "TranslatorCreateFromDeviceRequest" is not integer value.')

        if self.axes is not None:
            if not isinstance(self.axes, Iterable):
                raise ValueError('Property "Axes" of "TranslatorCreateFromDeviceRequest" is not iterable.')

            for i, axes_item in enumerate(self.axes):
                if axes_item is None:
                    raise ValueError(f'Item {i} in property "Axes" of "TranslatorCreateFromDeviceRequest" is None.')

                if not isinstance(axes_item, (int, float, decimal.Decimal)):
                    raise ValueError(f'Item {i} in property "Axes" of "TranslatorCreateFromDeviceRequest" is not a number.')

                if int(axes_item) != axes_item:
                    raise ValueError(f'Item {i} in property "Axes" of "TranslatorCreateFromDeviceRequest" is not integer value.')

        if self.config is not None:
            if not isinstance(self.config, TranslatorConfig):
                raise ValueError(f'Property "Config" of "TranslatorCreateFromDeviceRequest" is not an instance of "TranslatorConfig".')

            self.config.validate()



# This file is generated. Do not modify by hand.
# pylint: disable=line-too-long, unused-argument, f-string-without-interpolation, too-many-branches, too-many-statements, unnecessary-pass
from dataclasses import dataclass
from typing import Any, Dict, List
from collections.abc import Iterable
import zaber_bson
from .pvt_sequence_data import PvtSequenceData


@dataclass
class PvtCsvData:
    """
    Data representing content loaded from PVT CSV file, with sequence data and sequence names.
    """

    sequence_data: PvtSequenceData
    """
    The positions, velocities and times of the PVT sequence.
    """

    series_names: List[str]
    """
    The names of the columns in the CSV header.
    If the header columns do not contain names, these will default to `Series 1`, `Series 2`, etc..
    """

    @staticmethod
    def zero_values() -> 'PvtCsvData':
        return PvtCsvData(
            sequence_data=PvtSequenceData.zero_values(),
            series_names=[],
        )

    @staticmethod
    def from_binary(data_bytes: bytes) -> 'PvtCsvData':
        """" Deserialize a binary representation of this class. """
        data = zaber_bson.loads(data_bytes)  # type: Dict[str, Any]
        return PvtCsvData.from_dict(data)

    def to_binary(self) -> bytes:
        """" Serialize this class to a binary representation. """
        self.validate()
        return zaber_bson.dumps(self.to_dict())  # type: ignore

    def to_dict(self) -> Dict[str, Any]:
        return {
            'sequenceData': self.sequence_data.to_dict(),
            'seriesNames': [str(item or '') for item in self.series_names] if self.series_names is not None else [],
        }

    @staticmethod
    def from_dict(data: Dict[str, Any]) -> 'PvtCsvData':
        return PvtCsvData(
            sequence_data=PvtSequenceData.from_dict(data.get('sequenceData')),  # type: ignore
            series_names=data.get('seriesNames'),  # type: ignore
        )

    def validate(self) -> None:
        """" Validates the properties of the instance. """
        if self.sequence_data is None:
            raise ValueError(f'Property "SequenceData" of "PvtCsvData" is None.')

        if not isinstance(self.sequence_data, PvtSequenceData):
            raise ValueError(f'Property "SequenceData" of "PvtCsvData" is not an instance of "PvtSequenceData".')

        self.sequence_data.validate()

        if self.series_names is not None:
            if not isinstance(self.series_names, Iterable):
                raise ValueError('Property "SeriesNames" of "PvtCsvData" is not iterable.')

            for i, series_names_item in enumerate(self.series_names):
                if series_names_item is not None:
                    if not isinstance(series_names_item, str):
                        raise ValueError(f'Item {i} in property "SeriesNames" of "PvtCsvData" is not a string.')



# This file is generated. Do not modify by hand.
# pylint: disable=line-too-long, unused-argument, f-string-without-interpolation, too-many-branches, too-many-statements, unnecessary-pass
from dataclasses import dataclass
from typing import Any, Dict, List
from collections.abc import Iterable
import zaber_bson
from .measurement_sequence import MeasurementSequence


@dataclass
class PvtSequenceData:
    """
    Data representing a sequence of pvt points with defined positions, velocities and times.
    """

    positions: List[MeasurementSequence]
    """
    Pvt sequence positions for each axis.
    """

    velocities: List[MeasurementSequence]
    """
    Pvt velocities for each axis.
    """

    times: MeasurementSequence
    """
    Relative times corresponding to points in PVT sequence.
    """

    @staticmethod
    def zero_values() -> 'PvtSequenceData':
        return PvtSequenceData(
            positions=[],
            velocities=[],
            times=MeasurementSequence.zero_values(),
        )

    @staticmethod
    def from_binary(data_bytes: bytes) -> 'PvtSequenceData':
        """" Deserialize a binary representation of this class. """
        data = zaber_bson.loads(data_bytes)  # type: Dict[str, Any]
        return PvtSequenceData.from_dict(data)

    def to_binary(self) -> bytes:
        """" Serialize this class to a binary representation. """
        self.validate()
        return zaber_bson.dumps(self.to_dict())  # type: ignore

    def to_dict(self) -> Dict[str, Any]:
        return {
            'positions': [item.to_dict() for item in self.positions] if self.positions is not None else [],
            'velocities': [item.to_dict() for item in self.velocities] if self.velocities is not None else [],
            'times': self.times.to_dict(),
        }

    @staticmethod
    def from_dict(data: Dict[str, Any]) -> 'PvtSequenceData':
        return PvtSequenceData(
            positions=[MeasurementSequence.from_dict(item) for item in data.get('positions')],  # type: ignore
            velocities=[MeasurementSequence.from_dict(item) for item in data.get('velocities')],  # type: ignore
            times=MeasurementSequence.from_dict(data.get('times')),  # type: ignore
        )

    def validate(self) -> None:
        """" Validates the properties of the instance. """
        if self.positions is not None:
            if not isinstance(self.positions, Iterable):
                raise ValueError('Property "Positions" of "PvtSequenceData" is not iterable.')

            for i, positions_item in enumerate(self.positions):
                if positions_item is None:
                    raise ValueError(f'Item {i} in property "Positions" of "PvtSequenceData" is None.')

                if not isinstance(positions_item, MeasurementSequence):
                    raise ValueError(f'Item {i} in property "Positions" of "PvtSequenceData" is not an instance of "MeasurementSequence".')

                positions_item.validate()

        if self.velocities is not None:
            if not isinstance(self.velocities, Iterable):
                raise ValueError('Property "Velocities" of "PvtSequenceData" is not iterable.')

            for i, velocities_item in enumerate(self.velocities):
                if velocities_item is None:
                    raise ValueError(f'Item {i} in property "Velocities" of "PvtSequenceData" is None.')

                if not isinstance(velocities_item, MeasurementSequence):
                    raise ValueError(f'Item {i} in property "Velocities" of "PvtSequenceData" is not an instance of "MeasurementSequence".')

                velocities_item.validate()

        if self.times is None:
            raise ValueError(f'Property "Times" of "PvtSequenceData" is None.')

        if not isinstance(self.times, MeasurementSequence):
            raise ValueError(f'Property "Times" of "PvtSequenceData" is not an instance of "MeasurementSequence".')

        self.times.validate()



# This file is generated. Do not modify by hand.
# pylint: disable=line-too-long, unused-argument, f-string-without-interpolation, too-many-branches, too-many-statements, unnecessary-pass
from dataclasses import dataclass
from typing import Any, Dict
import zaber_bson


@dataclass
class TestRequest:

    return_error: bool = False

    data_ping: str = ""

    return_error_with_data: bool = False

    @staticmethod
    def zero_values() -> 'TestRequest':
        return TestRequest(
            return_error=False,
            data_ping="",
            return_error_with_data=False,
        )

    @staticmethod
    def from_binary(data_bytes: bytes) -> 'TestRequest':
        """" Deserialize a binary representation of this class. """
        data = zaber_bson.loads(data_bytes)  # type: Dict[str, Any]
        return TestRequest.from_dict(data)

    def to_binary(self) -> bytes:
        """" Serialize this class to a binary representation. """
        self.validate()
        return zaber_bson.dumps(self.to_dict())  # type: ignore

    def to_dict(self) -> Dict[str, Any]:
        return {
            'returnError': bool(self.return_error),
            'dataPing': str(self.data_ping or ''),
            'returnErrorWithData': bool(self.return_error_with_data),
        }

    @staticmethod
    def from_dict(data: Dict[str, Any]) -> 'TestRequest':
        return TestRequest(
            return_error=data.get('returnError'),  # type: ignore
            data_ping=data.get('dataPing'),  # type: ignore
            return_error_with_data=data.get('returnErrorWithData'),  # type: ignore
        )

    def validate(self) -> None:
        """" Validates the properties of the instance. """
        if self.data_ping is not None:
            if not isinstance(self.data_ping, str):
                raise ValueError(f'Property "DataPing" of "TestRequest" is not a string.')



# This file is generated. Do not modify by hand.
# pylint: disable=line-too-long, unused-argument, f-string-without-interpolation, too-many-branches, too-many-statements, unnecessary-pass
from dataclasses import dataclass
from typing import Any, Dict
import decimal
import zaber_bson


@dataclass
class TriggerEnableRequest:

    interface_id: int = 0

    device: int = 0

    trigger_number: int = 0

    count: int = 0

    @staticmethod
    def zero_values() -> 'TriggerEnableRequest':
        return TriggerEnableRequest(
            interface_id=0,
            device=0,
            trigger_number=0,
            count=0,
        )

    @staticmethod
    def from_binary(data_bytes: bytes) -> 'TriggerEnableRequest':
        """" Deserialize a binary representation of this class. """
        data = zaber_bson.loads(data_bytes)  # type: Dict[str, Any]
        return TriggerEnableRequest.from_dict(data)

    def to_binary(self) -> bytes:
        """" Serialize this class to a binary representation. """
        self.validate()
        return zaber_bson.dumps(self.to_dict())  # type: ignore

    def to_dict(self) -> Dict[str, Any]:
        return {
            'interfaceId': int(self.interface_id),
            'device': int(self.device),
            'triggerNumber': int(self.trigger_number),
            'count': int(self.count),
        }

    @staticmethod
    def from_dict(data: Dict[str, Any]) -> 'TriggerEnableRequest':
        return TriggerEnableRequest(
            interface_id=data.get('interfaceId'),  # type: ignore
            device=data.get('device'),  # type: ignore
            trigger_number=data.get('triggerNumber'),  # type: ignore
            count=data.get('count'),  # type: ignore
        )

    def validate(self) -> None:
        """" Validates the properties of the instance. """
        if self.interface_id is None:
            raise ValueError(f'Property "InterfaceId" of "TriggerEnableRequest" is None.')

        if not isinstance(self.interface_id, (int, float, decimal.Decimal)):
            raise ValueError(f'Property "InterfaceId" of "TriggerEnableRequest" is not a number.')

        if int(self.interface_id) != self.interface_id:
            raise ValueError(f'Property "InterfaceId" of "TriggerEnableRequest" is not integer value.')

        if self.device is None:
            raise ValueError(f'Property "Device" of "TriggerEnableRequest" is None.')

        if not isinstance(self.device, (int, float, decimal.Decimal)):
            raise ValueError(f'Property "Device" of "TriggerEnableRequest" is not a number.')

        if int(self.device) != self.device:
            raise ValueError(f'Property "Device" of "TriggerEnableRequest" is not integer value.')

        if self.trigger_number is None:
            raise ValueError(f'Property "TriggerNumber" of "TriggerEnableRequest" is None.')

        if not isinstance(self.trigger_number, (int, float, decimal.Decimal)):
            raise ValueError(f'Property "TriggerNumber" of "TriggerEnableRequest" is not a number.')

        if int(self.trigger_number) != self.trigger_number:
            raise ValueError(f'Property "TriggerNumber" of "TriggerEnableRequest" is not integer value.')

        if self.count is None:
            raise ValueError(f'Property "Count" of "TriggerEnableRequest" is None.')

        if not isinstance(self.count, (int, float, decimal.Decimal)):
            raise ValueError(f'Property "Count" of "TriggerEnableRequest" is not a number.')

        if int(self.count) != self.count:
            raise ValueError(f'Property "Count" of "TriggerEnableRequest" is not integer value.')
